package com.kodcu;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.TypedQuery;

/**
 * Hello world!
 *
 */
public class App
{
    public static void main( String[] args )
    {
        Logger logger=Logger.getLogger("JPQL Logs");
        
        EntityManagerFactory emf=Persistence.createEntityManagerFactory("JPQL_PU");
        EntityManager em=emf.createEntityManager();
        EntityTransaction trx= em.getTransaction();
        
        YayinEvi yayinEvi=new YayinEvi();
        yayinEvi.setYayineviAdi("Papatya Yayıncılık");
        
        Kitap kitap1=new Kitap();
        kitap1.setKitapAdi("Java ve Yazılım Tasarımı");
        kitap1.setFiyat(40d);
        kitap1.setYayinevi(yayinEvi);

        
        Kitap kitap2=new Kitap();
        kitap2.setKitapAdi("Java Mimarisiyle Kurumsal Çözümler");
        kitap2.setFiyat(20d);
     // Left Join testi için aşağıdaki satırı devredışı bırak 
        kitap2.setYayinevi(yayinEvi);
        
        List<Kitap> kitaplar=new ArrayList<Kitap>();
        kitaplar.add(kitap1);
        kitaplar.add(kitap2);
        
        yayinEvi.setKitaplar(kitaplar);
        
        for(int i=0;i<10;i++)
        try {
            trx.begin();
            em.persist(yayinEvi);
            em.flush();
            trx.commit();
        } catch (Throwable e) {
            System.out.println("İstisna oluştu,"
                    + " değişiklikler geri sarılacak..\n"+e.getMessage());
            trx.rollback();
        }
        
        /// SELECT, FROM : Tüm Kitap entity listesi
        
        logger.info("SELECT k FROM Kitap k");
        
        Query kitapSorgusu1 = em.createQuery("SELECT k FROM Kitap k");
        
        List<Object> kitapSorgusuListesi1 = kitapSorgusu1.getResultList();

        for(Object li:kitapSorgusuListesi1)
            logger.log(Level.INFO, "Tüm kitaplar sorgusu 1 : {0}", (Kitap)li);
        
        /// SELECT, FROM : Tüm Kitap entity listesi (TypedQuery ile)
        
        logger.info("SELECT k FROM Kitap k");
        
        TypedQuery<Kitap> kitapSorgusu2 =
                            em.createQuery("SELECT k FROM Kitap k", Kitap.class);

        List<Kitap> kitapSorgusuListesi2 = kitapSorgusu2.getResultList();
        
        for(Kitap ki:kitapSorgusuListesi2)
            logger.log(Level.INFO, "Tüm kitaplar sorgusu 2 : {0}", ki);
        
        /// COUNT tümcesi : Toplam Kitap entity sayısı
        
        logger.info("select COUNT(k) from Kitap k");
        
        Query countSorgu = em.createQuery("select COUNT(k) from Kitap k");
        
        Long toplam = (Long)countSorgu.getSingleResult();
        
        logger.log(Level.INFO, "Toplam kitap entity sayısı : {0}", toplam); 
        
        /// MAX tümcesi : En yüksek kitap fiyatı
        
        logger.info("select MAX(k.fiyat) from Kitap k");
        
        Query maxSorgu=em.createQuery("select MAX(k.fiyat) from Kitap k");
        
        logger.log(Level.INFO, "Max Sorgusu : {0}", (Double)maxSorgu.getSingleResult());
        
        /// WHERE tümcesi : Kitap fiyatı 30TL' den yüksek tüm kitaplar
        
        logger.info("select k from Kitap k where k.fiyat>30");
        
        Query buyukturSorgusu =
                    em.createQuery("select k from Kitap k where k.fiyat>30");
        
        List otuzdanBuyuklerListesi=buyukturSorgusu.getResultList();
        
        logger.info("Otuzdan büyük kitaplar ");
        
        for(Object otuz:otuzdanBuyuklerListesi)
        {
           Kitap kitap=(Kitap)otuz;
            System.out.println("->"+ kitap);
        }
        
        /// Çoklu sorgu
        
        logger.info("select k.id,k.kitapAdi from Kitap k");
        
        Query baziSorgusu =
                     em.createQuery("select k.id,k.kitapAdi from Kitap k");
        
        List<Object[]> baziKitaplar=baziSorgusu.getResultList();
        
        for(Object[] ki:baziKitaplar)
            logger.log(Level.INFO, "-> {0} -> {1}", new Object[]{ki[0], ki[1]});
        
        /// Çoklu kaynaklardan çoklu sorgu
        
        logger.info("select k, y from Kitap k,YayinEvi y");

        Query hemOndanHemSundan =
                    em.createQuery("select k, y from Kitap k,YayinEvi y");
        
        List<Object[]> hemOHemSuListesi = hemOndanHemSundan.getResultList();

        for(Object[] o:hemOHemSuListesi)
            for(Object oo:o)
            {
                if (oo instanceof Kitap)
                    logger.log(Level.INFO, "?> {0}", (Kitap)oo);
                if(oo  instanceof YayinEvi)
                    logger.log(Level.INFO, "?> {0}", (YayinEvi)oo);
            }
        
        /// GROUP BY ifadesi
        
        logger.info("SELECT y FROM YayinEvi y GROUP BY y.yayineviAdi");
        
        TypedQuery<YayinEvi> grupSorgusu =
        em.createQuery("SELECT y FROM YayinEvi y GROUP BY y.yayineviAdi", YayinEvi.class);
        
        List<YayinEvi> grupListesi = grupSorgusu.getResultList();
        
        for(YayinEvi li:grupListesi)
            logger.log(Level.INFO, "Grup çıktısı : {0}", li);
        
        /// NEW tümcesi
        
        logger.info("SELECT NEW "+
                "com.usta.YeniKitap(k.id,k.kitapAdi,k.fiyat,k.yayinevi)"+
                " FROM Kitap k ");
   
        Query newSorgu1 =
                em.createQuery("SELECT NEW "+
                "com.usta.YeniKitap(k.id,k.kitapAdi,k.fiyat,k.yayinevi)"+
                " FROM Kitap k ");
        
        List<Object> newSorgu1Listesi=newSorgu1.getResultList();
        
        for(Object obj :newSorgu1Listesi)
        logger.log(Level.INFO, "YeniKitap sorgu1 : {0}", (YeniKitap)obj);
        
        /// NEW tümcesi ( id belirle )
        
//        logger.info("SELECT NEW "
//                    +"com.usta.YeniKitap(k.id,k.kitapAdi,k.fiyat,k.yayinevi)"
//                    +"FROM Kitap k  WHERE k.id=2");
//          
//        Query newSorgu2 =
//        em.createQuery("SELECT NEW "
//                    +"com.usta.YeniKitap(k.id,k.kitapAdi,k.fiyat,k.yayinevi)"
//                    +"FROM Kitap k  WHERE k.id=2");
//        
//        Object newSorgu1TekEntity=newSorgu2.getSingleResult();
//        
//        logger.log(Level.INFO, "YeniKitap sorgu2 : {0}", (YeniKitap)newSorgu1TekEntity);
        
       /// DISTINCT tümcesi
        
        logger.info("select DISTINCT NEW "
                + "com.usta.YeniKitap(k.id,k.kitapAdi,k.fiyat,k.yayinevi)"
                + "from Kitap k GROUP BY k.kitapAdi");
        
        TypedQuery<YeniKitap> newSorgusu =
                em.createQuery("select DISTINCT NEW "
                + "com.usta.YeniKitap(k.id,k.kitapAdi,k.fiyat,k.yayinevi)"
                + "from Kitap k GROUP BY k.kitapAdi",YeniKitap.class);
        
        List<YeniKitap> newListesi = newSorgusu.getResultList();
        
        for(YeniKitap yeni:newListesi)
            logger.log(Level.INFO, "->{0}", yeni);
        
        /// INNER JOIN veya JOIN tümcesi
        
        logger.info("select k,y from Kitap k "
                               +" INNER JOIN k.yayinevi y");
        
        Query katilimSorgusu =
                em.createQuery("select k,y from Kitap k "
                               +" INNER JOIN k.yayinevi y");
        
        List<Object[]> katilimListesi = katilimSorgusu.getResultList();
        
        for(Object[] oo:katilimListesi)
            logger.log(Level.INFO, "->{0}->{1}", new Object[]{oo[0], oo[1]});
        
        /// LEFT OUTER JOIN veya LEFT JOIN
        
        logger.info("select k,y from Kitap k "
                                +"LEFT OUTER JOIN k.yayinevi y");

        Query katilimLeftSorgusu =
                em.createQuery("select k,y from Kitap k "
                                +"LEFT OUTER JOIN k.yayinevi y");
        
        List<Object[]> katilimLeftListesi =
                            katilimLeftSorgusu.getResultList();
        
        for(Object[] oo:katilimLeftListesi)
        logger.log(Level.INFO, "-> {0}-> {1}", new Object[]{oo[0], oo[1]});
        

        /// Eklenen tüm kayıtları temizle.. 
        
//       trx.begin();
//        int nekadarSilindi=em.createQuery("DELETE FROM Kitap k").executeUpdate();
//        System.out.println("Ne kadar kayıt silindi? : "+nekadarSilindi);
//       trx.commit();

        
    }
}